#################################################################################
# Create Dll Wrapper
# 
#  Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia. 
#  All Rights Reserved.
# 
#################################################################################

@interfaceNames = 
(
	"Physics", 
	"DynamicObject", 
	"DynamicObjectCar", 
	"StaticObject",
	"PhysicsJoint",
	"Collision",
	"CollisionObject",
	"SurfaceAttribute", 
	"CollisionObjectCombo", 
	"CollisionObjectComplex",
	"CollisionObjectAABBMesh",
	"CollisionObject"
);
@otherNames = 
(
	"AABB", 
	"Vec3", 
	"Mat33", 
	"MFrame", 
	"MFrame", 
	"Char", 
	"PhysicsJoint", 
	"LineIntersection", 
	"EulerAngles",
	"PreProcessCollisionCallBack",
	"PostProcessCollisionCallBack",
	"DynamicVsDynamicCollisionTestCallBack",
	"DynamicVsStaticCollisionTestCallBack"
);
$TA_EXPORT = "TA_EXPORT";
$TA_CALL = "TA_CALL";

convert('../TA/Physics/DynamicObject.h', './Physics/', 'DynamicObject');
convert('../TA/Physics/DynamicObjectCar.h', './Physics/', 'DynamicObjectCar');
convert('../TA/Physics/Physics.h', './Physics/', 'Physics');
convert('../TA/Physics/StaticObject.h', './Physics/', 'StaticObject');
convert('../TA/Physics/Collision.h', './Physics/', 'Collision');
convert('../TA/Physics/CollisionObjectAABBMesh.h', './Physics/', 'CollisionObjectAABBMesh');

use File::Copy;

copyCommonFiles('../TA/Common/Vec2.h', './Common/', 'Vec2.h');
copyCommonFiles('../TA/Common/Vec3.h', './Common/', 'Vec3.h');
copyCommonFiles('../TA/Common/Vec4.h', './Common/', 'Vec4.h');
copyCommonFiles('../TA/Common/Vector.h', './Common/', 'Vector.h');
copyCommonFiles('../TA/Common/Matrix.h', './Common/', 'Matrix.h');
copyCommonFiles('../TA/Common/MFrame.h', './Common/', 'MFrame.h');
copyCommonFiles('../TA/Common/EulerAngles.h', './Common/', 'EulerAngles.h');
copyCommonFiles('../TA/Common/Maths.h', './Common/', 'Maths.h');
copyCommonFiles('../TA/Common/AABB.h', './Common/', 'AABB.h');
copyCommonFiles('../TA/Common/Common.h', './Common/', 'Common.h');
copyCommonFiles('../TA/Common/Debug.h', './Common/', 'Debug.h');

#################################################################################
# subs
#################################################################################
sub copyCommonFiles
{
	my $in = @_[0];
	my $outDir = @_[1];
	my $outFile = @_[2];
	my $out = $outDir.$outFile;
	copy($in, $out);
	
	open(IN, $in); 
	
	while ($line = <IN>) 
	{	
		# Find a class to export
		if ($line =~ m/\#include \"(.*)\"/)
		{
			my $inlineFile = $1;
			if ($inlineFile =~	m/\.inl/)
			{
				copy('../TA/Common/'.$inlineFile, $outDir.$inlineFile);
			}
		}
	}
	
}

sub convert
{
	my $in = @_[0];
	my $outDir = @_[1];
	my $outFile = @_[2];
	my $out = $outDir.$outFile;
	open(IN, $in); 
	open(OUT_INL, ">$out.inl"); 
	open(OUT_HPP, ">$out.hpp"); 
	open(OUT_C, ">$out"."_c.cpp"); 
	open(OUT_H, ">$out.h"); 
	open(OUT_PAS, ">$outDir"."Ta$outFile.pas"); 	
	open(IMP_PAS,'>', \$pascalImplementation);
	
	$outUpper = $outFile;
	$outUpper =~ tr/[a-z]/[A-Z]/;
	
	print OUT_H '//---------------------------------------------------------------------------------'."\n";
	print OUT_H '//'."\n";
	print OUT_H '// Autogenerated from CreateDllWrapper.pl'."\n";
	print OUT_H '//'."\n";
	print OUT_H '// Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia.'."\n";
	print OUT_H '// All Rights Reserved.'."\n";
	print OUT_H '//'."\n";
	print OUT_H '//---------------------------------------------------------------------------------'."\n\n";
	print OUT_HPP '//---------------------------------------------------------------------------------'."\n";
	print OUT_HPP '//'."\n";
	print OUT_HPP '// Autogenerated from CreateDllWrapper.pl'."\n";
	print OUT_HPP '//'."\n";
	print OUT_HPP '// Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia.'."\n";
	print OUT_HPP '// All Rights Reserved.'."\n";
	print OUT_HPP '//'."\n";
	print OUT_HPP '//---------------------------------------------------------------------------------'."\n\n";
	print OUT_C '//---------------------------------------------------------------------------------'."\n";
	print OUT_C '//'."\n";
	print OUT_C '// Autogenerated from CreateDllWrapper.pl'."\n";
	print OUT_C '//'."\n";
	print OUT_C '// Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia.'."\n";
	print OUT_C '// All Rights Reserved.'."\n";
	print OUT_C '//'."\n";
	print OUT_C '//---------------------------------------------------------------------------------'."\n\n";
	print OUT_INL '//---------------------------------------------------------------------------------'."\n";
	print OUT_INL '//'."\n";
	print OUT_INL '// Autogenerated from CreateDllWrapper.pl'."\n";
	print OUT_INL '//'."\n";
	print OUT_INL '// Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia.'."\n";
	print OUT_INL '// All Rights Reserved.'."\n";
	print OUT_INL '//'."\n";
	print OUT_INL '//---------------------------------------------------------------------------------'."\n\n";
	print OUT_PAS '{'."\n";
	print OUT_PAS ''."\n";
	print OUT_PAS ' Autogenerated from CreateDllWrapper.pl'."\n";
	print OUT_PAS ''."\n";
	print OUT_PAS ' Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia.'."\n";
	print OUT_PAS ' All Rights Reserved.'."\n";
	print OUT_PAS ''."\n";
	print OUT_PAS '}'."\n\n";
	
	print OUT_H "\#ifndef TA_$outUpper"."_WRAPPER_H\n";
	print OUT_H "\#define TA_$outUpper"."_WRAPPER_H\n\n";
	
	print OUT_HPP "\#ifndef TA_$outUpper"."_WRAPPER_HPP\n";
	print OUT_HPP "\#define TA_$outUpper"."_WRAPPER_HPP\n\n";
	print OUT_HPP "namespace TA\n{\n\n";	
	print OUT_INL "namespace TA\n{\n\n";
	
	print OUT_C "#include \"../../TA/Physics/$outFile.h\"\n\n";	
	print OUT_C "#define TA_C_IMPLEMENTATION\n\n";
	print OUT_C "#include \"$outFile.h\"\n\n";
	print OUT_C "using namespace TA;\n\n";	
				
	print OUT_H '#include "../Common/Types.h"'."\n\n";
	print OUT_H '#ifdef __cplusplus'."\n";
	print OUT_H 'extern "C" {'."\n";
	print OUT_H '#endif // __cplusplus'."\n\n";
	
	print OUT_PAS "unit Ta$outFile;\n\n";	
	print OUT_PAS "interface\n\n";
	print OUT_PAS "uses TaTypes;\n\n";
	
	my @functionNameArray = ();
	my $ifdef = 0;
	my $className = "";
	my $classNameForC = "";
	my $classNameOriginal = "";
	my $exportClass = 0;
	#my @pascalImplementationArray = ();
	my $pascalImpCount = 0;
	while ($line = <IN>) 
	{	
		# Find a class to export
		if ($line =~ m/pl\.BeginExportClass\((.*)\)/)
		{		
			@split = split(/:/, $1);
			$classNameOriginal = @split[0];
			$classNameOriginal =~ s/\s//g;
			$className = $classNameOriginal ;
			$classNameForC = "Ta".$classNameOriginal;			
			print OUT_HPP "class $className";
			if (@split[1])
			{
				print OUT_HPP ":@split[1]";
			}
			print OUT_HPP "\n";
			print OUT_HPP "{\n";
			print OUT_HPP "public\:\n\n";
			if (!@split[1])
			{
				#print OUT_HPP "\tvoid* m_pInst;\n\n";
			}
			$exportClass = 1;			
		}
		else
		{
			if ($ifdef)
			{
				if ($line =~ /#endif/)
				{
					$ifdef = 0
				}				
			}
			else
			{
				if ($line =~ /#ifdef\s+TA_PHYSICS_INTERNAL/)
				{
					$ifdef = 1;
				}
				if ($line =~ /#ifndef\s+TA_WRAPPER/)
				{
					$ifdef = 1;
				}
			}
			if (!$ifdef)
			{
				if ($exportClass)
				{
					# Find end of class export
					if ($line =~ /\/\/pl\./)
					{
						# Process perl script functions
						if ($line =~ /pl\.EndExportClass(\(.*\))/)
						{		
							if ($1 =~ /\(Collision\)/)
							{
								# handle collision as a special case
								print OUT_HPP "\t$className() {};\n\n";
								print OUT_HPP "private\:\n\n";
								print OUT_HPP "\tu8 data[128];";
							}
							else
							{
								print OUT_HPP "private\:\n\n";
								print OUT_HPP "\t$className();\n\n";
								print OUT_HPP "\t$className(const $className& that);\n\n";
							}
							print OUT_HPP "};\n\n";
							$className = "";
							$exportClass = 0;
						}
						if ($line =~ /pl\.InheritFrom\((.*)\)/)
						{		
							print OUT_H "$TA_EXPORT Ta$1* $TA_CALL $classNameForC"."_GetTa$1(\n\t$classNameForC* p$classNameForC);\n\n";
							
							print OUT_C "$TA_EXPORT Ta$1* $TA_CALL $classNameForC"."_GetTa$1(\n\t$classNameForC* p$classNameForC)\n";
							print OUT_C "{\n";
							print OUT_C "\treturn (Ta$1*)p$classNameForC;\n";
							print OUT_C "}\n";
							
							print OUT_PAS "function $classNameForC"."_GetTa$1(\n\tp$classNameForC\: $classNameForC\_Ptr) \: Ta$1\_Ptr; stdcall;\n\n";
							print IMP_PAS "function $classNameForC"."_GetTa$1(\n\tp$classNameForC\: $classNameForC\_Ptr) \: Ta$1\_Ptr; stdcall;\n";
							print IMP_PAS "external \'TaPhysics.dll\' name ";
							print IMP_PAS "\'_$classNameForC"."_GetTa$1\@4\';\n\n";
						}
						if ($line =~ /pl\.Release\(.*\)/)
						{		
							print OUT_H "$TA_EXPORT void $TA_CALL $classNameForC"."_Release(\n\t$classNameForC* p$classNameForC);\n\n";
							
							print OUT_C "$TA_EXPORT void $TA_CALL $classNameForC"."_Release(\n\t$classNameForC* p$classNameForC)\n";
							print OUT_C "{\n";
							print OUT_C "\treturn (($classNameOriginal*)p$classNameForC)->Release();\n";
							print OUT_C "}\n";
							
							print OUT_HPP "\tvoid Release();\n\n";
							
							print OUT_INL "void $className\:\:Release()\n";
							print OUT_INL "{\n";
							print OUT_INL "\t$classNameForC"."_Release(\n\t\t($classNameForC*)this);\n";
							print OUT_INL "}\n";
							
							print OUT_PAS "procedure $classNameForC"."_Release(\n\tp$classNameForC\: $classNameForC\_Ptr); stdcall;\n\n";
							print IMP_PAS "procedure $classNameForC"."_Release(\n\tp$classNameForC\: $classNameForC\_Ptr); stdcall;\n";
							print IMP_PAS "external \'TaPhysics.dll\' name ";
							print IMP_PAS "\'_$classNameForC"."_Release\@4\';\n\n";
						}
					}
					else
					{
						# Process C++ function definitions
						if ($line =~ /\benum\b/ )
						{
							print OUT_PAS "const\n";
							print OUT_H "enum\n{\n";
							print OUT_HPP "\tenum\n\t{\n";
							$line = <IN>;
							$line = <IN>;
							while($line !~ m/[}]/)
							{
								print OUT_HPP "$line";
								$line =~ s/\b([A-Z]\w*)/TA_$outUpper\_$1/g; # add prefix to enums
								$line =~ s/^[\t ]*//; # remove leading white spaces
								print OUT_H "\t$line";
										
								$line =~ s/=.*pascal\((.*)\)/= $1/g;						
								$line =~ s/,/;/g;			
								$line =~ s/0x/\$/g;
								$line =~ s/=/: Integer =/g;
								$line =~ s/\s+/ /g;
								$line =~ s/\|/or/g;
								print OUT_PAS "\t$line\n";
								$line = <IN>
							}	
							print OUT_PAS "\n";
							print OUT_H '};'."\n\n";	
							print OUT_HPP "\t};"."\n\n";				
						}
						
						# Process C++ function definitions
						$function = "";					
						if ($line =~ m/\(/)
						{
							while($line !~ m/[;{]/)
							{
								$function = $function.$line;
								$line = <IN>
							}
							$function = $function.$line;
							$function =~ s/\n//g;
							replaceNames($function);
						}
						
						$function =~ s/[\{\;].*//; # remove everything after function definition
							
						if ($function =~ m/(.*?)\((.*)\)(.*)/)
						{
							$funcitonName = $1;
							$params = $2;
							$functionTail = $3;
							
							$constFunction = 0;	
							if ($functionTail =~ m/(\S....)/)	
							{
								if ($1 eq "const")
								{
									$constFunction = 1;
								}
							}
												
							$funcitonName =~ s/virtual\s//; # remove virtual from function def						
							
							
						
							
							@charArray = split(//, $params);
							$depth = 0;
							for ($i = 0; $i < $#charArray; $i++)
							{
								if (@charArray[$i] eq "(")
								{
									$depth++;
								}
								if (@charArray[$i] eq ")")
								{
									$depth--;
								}
								if ($depth > 0 && (@charArray[$i] eq ","))
								{
									$charArray[$i] = "@";
								}
							}
							$params = join('', @charArray);
								
							@splitParams = split(/,/, $params);
							my @splitParamsDefaltValue = ();
							$numParams = @splitParams;	
							for ($param = 0; $param < $numParams; $param++)
							{							
								@tmpSplit = split(/\s*=\s*/, @splitParams[$param]);
								@splitParams[$param] = @tmpSplit[0];
								@splitParamsDefaltValue[$param] =  @tmpSplit[1];
								@splitParamsDefaltValue[$param]  =~ s/@/,/g;
								@splitParams[$param] =~ s/\s*//;
							}		
							
							@functionNameSplit = split(/\s/, $funcitonName);
							$functionNameCount = 1;
							for ($i = 0; $i <= $#functionNameArray; $i++)
							{
								if (@functionNameArray[$i] eq @functionNameSplit[$#functionNameSplit])
								{
									$functionNameCount++;
								}
							}
							push(@functionNameArray, @functionNameSplit[$#functionNameSplit]);
							if ($functionNameCount == 1)
							{
								$functionNameCount = "";
							}
												
							#print "--------------------------------------------------------------------------\n";
							#print "C++ header\n";
							#print "--------------------------------------------------------------------------\n";	
							my $cppFuncitonName = $funcitonName;
							$cppFuncitonName =~ s/TAC_CALL/TA_CALL/g;
							print OUT_HPP "$cppFuncitonName(";
							for ($param = 0; $param < $numParams; $param++)
							{
								print OUT_HPP "\n\t\t@splitParams[$param]";
								if (@splitParamsDefaltValue[$param])
								{
									print OUT_HPP " = @splitParamsDefaltValue[$param]";
								}
								if ($param < $numParams - 1)
								{
									print OUT_HPP ",";
								}
							}
							print OUT_HPP ")";
							if ($constFunction)
							{
								print OUT_HPP " const";
							}
							print OUT_HPP ";\n\n";
							
							#print "--------------------------------------------------------------------------\n";
							#print "C++ implementation\n";
							#print "--------------------------------------------------------------------------\n";	
							
							$staticFunction = 0;
							$needReturn = 1;
								
						
							print OUT_INL "inline ";

							my $count = @functionNameSplit;
							for ($i = 1; $i < $count - 1; $i++)
							{
								if (@functionNameSplit[$i] eq "static")
								{
									$staticFunction = 1;
								}
								else
								{
									if (!(@functionNameSplit[$i] eq "TAC_CALL"))
									{
										print OUT_INL "@functionNameSplit[$i] ";
										if (@functionNameSplit[$i] eq "void")
										{
											$needReturn = 0;
										}
									}
								}
							}
							print OUT_INL "$classNameOriginal\:\:";
							print OUT_INL "@functionNameSplit[$count - 1](";
							for ($param = 0; $param < $numParams; $param++)
							{
								print OUT_INL "\n\t@splitParams[$param]";
								if ($param < $numParams - 1)
								{
									print OUT_INL ",";
								}
							}
							print OUT_INL ")";
							if ($constFunction)
							{
								print OUT_INL " const";
							}
							print OUT_INL "\n{\n";
							print OUT_INL "\t";
							if ($needReturn)
							{
								print OUT_INL "return ";
								
								if ($funcitonName =~ /\bbool\b/)
								{
									print OUT_INL '!!';
								}
								else
								{
									if (paramNeedsCast($funcitonName))
									{
										print OUT_INL '(';
										my $first = 1;
										for (my $i = 1; $i < $#functionNameSplit; $i++)
										{
											my $thisBit = @functionNameSplit[$i];
											if ($thisBit !~ /static/ &&
												$thisBit !~ /TAC_CALL/)
											{
												if ($first)
												{
													$first = 0;
												}
												else
												{
													$first = 1;
													print OUT_INL " ";
												}
												print OUT_INL $thisBit;	
											}
										}
										if ($funcitonName !~ /[&\*]/)
										{
											print OUT_INL '&'; # add a reference to the cast if we don't have one
										}
										print OUT_INL ')';
									}	
									if ($funcitonName =~ /&/)
									{
										print OUT_INL "*"; # convert from a pointer to a reference
									}
								}
								
							}
							print OUT_INL "$classNameForC\_@functionNameSplit[$count - 1]$functionNameCount(";
							if (!$staticFunction)
							{
								if ($numParams > 0)
								{
									print OUT_INL "\n\t\t($classNameForC*)this,";
								}
								else
								{
									print OUT_INL "($classNameForC*)this";
								}
							}
							for ($param = 0; $param < $numParams; $param++)
							{
								@thisSplitParam = split(/\s/, @splitParams[$param]);
								print OUT_INL "\n\t\t";
								# do we need to cast the paramater?
								if (paramNeedsCast(@splitParams[$param]))
								{
									print OUT_INL '(';
									for (my $i = 0; $i < $#thisSplitParam; $i++)
									{
										my $thisBit = @thisSplitParam[$i];
										$thisBit =~ s/&/*/g; # convert references to pointers
										replaceNamesForC($thisBit);
										print OUT_INL $thisBit;	
										if ($i < $#thisSplitParam - 1)
										{
											print OUT_INL " ";
										}
									}
									print OUT_INL ')';
								}	
								if (@splitParams[$param] =~ /&/)
								{
									print OUT_INL "&"; # convert from a reference to a pointer
								}							
								print OUT_INL "@thisSplitParam[$#thisSplitParam]";
								if ($param < $numParams - 1)
								{
									print OUT_INL ",";
								}
							}
							print OUT_INL ");\n";						
							print OUT_INL "}\n\n";								
								
								
								
							replaceNamesForC($funcitonName);
							replaceNamesForC($params);						
							$size = @splitParams;
							for ($i = 0; $i < $size; $i++)
							{
								replaceNamesForC(@splitParams[$i])
							}
							$size = @functionNameSplit;
							for ($i = 0; $i < $size; $i++)
							{
								replaceNamesForC(@functionNameSplit[$i])
							}		
						
							
							
							#print "--------------------------------------------------------------------------\n";
							#print "C header\n";
							#print "--------------------------------------------------------------------------\n";	
							my $count = @functionNameSplit;
							$staticFunction = 0;
							print OUT_H "$TA_EXPORT ";
							for ($i = 1; $i < $count - 1; $i++)
							{
								$replaceReferences = @functionNameSplit[$i];
								$replaceReferences =~ s/&/*/g; 
								if ($replaceReferences eq "void")
								{
									$needReturn = 0;
								}
								if ($replaceReferences eq "static")
								{
									$staticFunction = 1;
								}
								if (!($replaceReferences eq "static") &&
									!($replaceReferences eq "TAC_CALL"))
								{
									print OUT_H "$replaceReferences ";
								}						
							}
							print OUT_H "$TA_CALL ";
							print OUT_H $classNameForC."_";
							print OUT_H "@functionNameSplit[$count - 1]$functionNameCount(";
							if (!$staticFunction)
							{
								print OUT_H "\n\t";
								if ($constFunction)
								{
									print OUT_H "const ";
								}
								print OUT_H "$classNameForC* p$classNameForC";
								if ($numParams > 0)
								{
									print OUT_H ",";
								}
							}
							for ($param = 0; $param < $numParams; $param++)
							{
								$replaceReferences = @splitParams[$param];
								$replaceReferences =~ s/&/*/g; 
								print OUT_H "\n\t$replaceReferences";
								if ($param < $numParams - 1)
								{
									print OUT_H ",";
								}
								if (@splitParamsDefaltValue[$param])
								{
									print OUT_H " \/* default = @splitParamsDefaltValue[$param] *\/";
								}
							}
							print OUT_H ")";
							print OUT_H ";\n\n";
							
							
							
							#print "--------------------------------------------------------------------------\n";
							#print "C implementation\n";
							#print "--------------------------------------------------------------------------\n";	
							print OUT_C "$TA_EXPORT ";
							$count = @functionNameSplit;
							$staticFunction = 0;
							$needReturn = 1;
							for ($i = 1; $i < $count - 1; $i++)
							{
								$replaceReferences = @functionNameSplit[$i];
								$replaceReferences =~ s/&/*/g; 
								if ($replaceReferences eq "void")
								{
									$needReturn = 0;
								}
								if ($replaceReferences eq "static")
								{
									$staticFunction = 1;
								}
								if (!($replaceReferences eq "static") &&
									!($replaceReferences eq "TAC_CALL"))
								{
									print OUT_C "$replaceReferences ";
								}						
							}
							print OUT_C "$TA_CALL ";
							print OUT_C $classNameForC."_";
							print OUT_C "@functionNameSplit[$count - 1]$functionNameCount(";
							if (!$staticFunction)
							{
								print OUT_C "\n\t";
								if ($constFunction)
								{
									print OUT_C "const ";
								}
								print OUT_C "$classNameForC* p$classNameForC";
								if ($numParams > 0)
								{
									print OUT_C ",";
								}
							}
							for ($param = 0; $param < $numParams; $param++)
							{
								$replaceReferences = @splitParams[$param];
								$replaceReferences =~ s/&/*/g; 
								print OUT_C "\n\t$replaceReferences";
								if ($param < $numParams - 1)
								{
									print OUT_C ",";
								}
							}
							print OUT_C ")";
							print OUT_C "\n{\n";	
							print OUT_C "\t";
							if ($needReturn)
							{				
								print OUT_C "return ";
								if (hasReplacedName($funcitonName))
								{
									print OUT_C "(";
									$first = 1;
									for ($i = 1; $i < $#functionNameSplit ; $i++)
									{
										if (!(@functionNameSplit[$i] eq "static") &&
											!(@functionNameSplit[$i] eq "TAC_CALL"))
										{
											if ($first)
											{
												$first = 0;
											}
											else
											{
												print OUT_C " ";
											}
											$changedVar = @functionNameSplit[$i];																						
											$changedVar =~ s/&/*/;
											print OUT_C $changedVar;	
										}
									}	
									if ($changedVar =~ /TaBool/ ||
										$changedVar =~ /ta_u32/ ||
										$changedVar =~ /\*/)
									{
										print OUT_C ")";	
									}
									else
									{
										print OUT_C "&)";	
									}
									if ($funcitonName =~ /&/)
									{
										print OUT_C "&";
									}
										
								}	
							}
							if ($staticFunction)
							{
								print OUT_C "$classNameOriginal\:\:@functionNameSplit[$count - 1](";
							}
							else
							{
								print OUT_C "(($classNameOriginal*)p$classNameForC)->@functionNameSplit[$count - 1](";	
							}
							for ($param = 0; $param < $numParams; $param++)
							{
								# todo: remove default paramater values fi
								
								my @thisSplitParam = split(/\s/, @splitParams[$param]);
								print OUT_C "\n\t\t";	
								# cast parameter
								if (hasReplacedName(@splitParams[$param]))
								{
									print OUT_C "(";
									for ($i = 0; $i < $#thisSplitParam ; $i++)
									{
										$changedVar = @thisSplitParam[$i];
										$changedVar =~ s/\bTaBool/bool/;
										$changedVar =~ s/\bTa//;
										print OUT_C $changedVar;
										
										if ($i < $#thisSplitParam - 1)
										{
											print OUT_C " ";
										}
									}								
									if (@splitParams[$param] =~ /&/)
									{
										print OUT_C ")";
										print OUT_C "*"; # pointer to reference conversion
									}
									else
									{								
										print OUT_C "&)";
									}	
								}													
								print OUT_C "@thisSplitParam[$#thisSplitParam]";
								if ($param < $numParams - 1)
								{
									print OUT_C ",";
								}
							}
							print OUT_C ");\n";							
							print OUT_C "}\n";
							
							
							
							
							#print "--------------------------------------------------------------------------\n";
							#print ".PAS pascal file\n";
							#print "--------------------------------------------------------------------------\n";	

							my $count = @functionNameSplit;
							$staticFunction = 0;
							$needReturn = 1;
							# are we a function or procedure
							for ($i = 1; $i < $count - 1; $i++)
							{
								if (@functionNameSplit[$i] eq "void")
								{
									$needReturn = 0;
								}
								if ($functionNameSplit[$i] eq "static")
								{
									$staticFunction = 1;
								}
							}							
							if ($needReturn)
							{
								#@pascalImplementationArray[$pascalImpCount] = "function ";
								print OUT_PAS "function ";
								print IMP_PAS "function ";
							}
							else
							{
								#@pascalImplementationArray[$pascalImpCount] = "procedure ";
								print OUT_PAS "procedure ";
								print IMP_PAS "procedure ";
							}
							
							# function name							
							print OUT_PAS "$classNameForC\_@functionNameSplit[$count - 1]$functionNameCount(";
							print IMP_PAS "$classNameForC\_@functionNameSplit[$count - 1]$functionNameCount(";
							# class pointer as first paramater
							if (!$staticFunction)
							{
								print OUT_PAS "\n\t";
								print IMP_PAS "\n\t";
								if ($constFunction)
								{
									#print OUT_PAS "const ";
									#print IMP_PAS "const ";
								}
								print OUT_PAS "p$classNameForC: $classNameForC"."_Ptr";
								print IMP_PAS "p$classNameForC: $classNameForC"."_Ptr";
								if ($numParams > 0)
								{
									print OUT_PAS ";";
									print IMP_PAS ";";
								}
							}
							# other paramaters
							for ($param = 0; $param < $numParams; $param++)
							{
								$replaceRefAndPtr = @splitParams[$param];
								$replaceRefAndPtr =~ s/[&*]/_Ptr/g; 
								replaceNamesForPascal($replaceRefAndPtr);
								
								my @thisSplitParam = split(/\s/, $replaceRefAndPtr);
								print OUT_PAS "\n\t$thisSplitParam[$#thisSplitParam] \:";
								print IMP_PAS "\n\t$thisSplitParam[$#thisSplitParam] \:";
								for (my $i = 0; $i < $#thisSplitParam; $i++)
								{							
									if (!($thisSplitParam[$i] eq "const"))
									{
										# convert from C++ types to pascal types
										print OUT_PAS " $thisSplitParam[$i]";
										print IMP_PAS " $thisSplitParam[$i]";
									}
								}
								
								
								#print OUT_PAS "\n\t$replaceRefAndPtr";
								if ($param < $numParams - 1)
								{
									print OUT_PAS ";";
									print IMP_PAS ";";
								}
								if (@splitParamsDefaltValue[$param])
								{
									print OUT_PAS " { default = @splitParamsDefaltValue[$param] }";
									print IMP_PAS " { default = @splitParamsDefaltValue[$param] }";
								}
							}
							print OUT_PAS ")";
							print IMP_PAS ")";
							if ($needReturn)
							{
								print OUT_PAS " :";
								print IMP_PAS " :";
								for (my $i = 1; $i < $count - 1; $i++)
								{
									$replaceRefAndPtr = @functionNameSplit[$i];
									$replaceRefAndPtr =~ s/[&*]/_Ptr/g; 
									replaceNamesForPascal($replaceRefAndPtr);
									if (!($replaceRefAndPtr eq "static") &&
										!($replaceRefAndPtr eq "TAC_CALL") &&
										!($replaceRefAndPtr eq "const"))
									{
										print OUT_PAS " $replaceRefAndPtr";
										print IMP_PAS " $replaceRefAndPtr";
									}						
								}
							}
							print OUT_PAS "; stdcall;\n\n";
							print IMP_PAS "; stdcall;\nexternal \'TaPhysics.dll\' name ";
							my $numParamBytes = $numParams * 4; #todo: this is just a guess, may have to calculate properly	
							$numParamBytes += 4 if (!$staticFunction);
							print IMP_PAS "\'_$classNameForC\_@functionNameSplit[$count - 1]$functionNameCount\@$numParamBytes\';\n\n";
							$pascalImpCount++;							
							#print "\n\n\n\n";
						}	
					}
				}
			}
		}		
	}
	
	print OUT_H '#ifdef __cplusplus'."\n";
	print OUT_H '} // extern "C"'."\n";
	print OUT_H '#endif // __cplusplus'."\n\n";
	
	print OUT_H "#if defined(__cplusplus) && !defined(TA_C_IMPLEMENTATION)\n";
	print OUT_H "#include \"$outFile.hpp\"\n";
	print OUT_H "#endif // defined(__cplusplus) && !defined(TA_C_IMPLEMENTATION)\n\n";	
		
	print OUT_HPP "} \/\/ namespace TA\n\n";
	print OUT_HPP "\n#include \"$outFile.inl\"\n\n";
	
	print OUT_INL "\n} \/\/ namespace TA\n";
	
	print OUT_H "\#endif \/\/ TA_$outUpper"."WRAPPER_H\n";	
	print OUT_HPP "\#endif \/\/ TA_$outUpper"."_WRAPPER_HPP\n";
	
	# pascal implementation
	print OUT_PAS "implementation\n\n";
	#print OUT_PAS @pascalImplementationArray;
	close(IMP_PAS); 
	print OUT_PAS $pascalImplementation;
	print OUT_PAS "\nend.\n";
	
	
	close(IN);
	close(OUT_PAS); 
	close(OUT_INL);
	close(OUT_HPP);
	close(OUT_C);
	close(OUT_H);
	
	open(IN_PAS, "<$outDir"."Ta$outFile.pas"); 	
	open(TMP_PAS,'>', \$pascalTmp);	
	while ($line = <IN_PAS>)
	{
		if (!dontUsePascalRef($line))
		{	
			$line =~ s/\b([0-9a-zA-Z_]+\s*\:\s*[0-9a-zA-Z_]+)_Ptr/var $1/g;
		}
	
	#	it seams that you cannot return by reference in pascal		
	#	while ($line =~ /\:\s*([0-9a-zA-Z_]+)_Ptr; stdcall/g)
	#	{	
	#		$replace = $1;
	#		if (!dontUsePascalRef($replace))
	#		{
	#			$line =~ s/$replace\_Ptr; stdcall/var $replace; stdcall/g; 
	#		}
	#		#$line =~ s/\:\s*([0-9a-zA-Z_]+)_Ptr; stdcall/var $1; stdcall/g;
	#	}
			
		# change naming convensions to fit delphi better
		if ($line !~ /\s*unit\b/)
		{
			#put a T infront of Type names
			for (my $k = 0; $k <= $#interfaceNames; $k++)
			{
				$line =~ s/\bTa$interfaceNames[$k]\b/TTa$interfaceNames[$k]/g;
			}
			for (my $k = 0; $k <= $#otherNames; $k++)
			{
				$line =~ s/\bTa$otherNames[$k]\b/TTa$otherNames[$k]/g;
			}
			
			# change from "<PointerName>_Ptr" to P<PointerName>
			for (my $k = 0; $k <= $#interfaceNames; $k++)
			{
				$line =~ s/\bTa$interfaceNames[$k]_Ptr(;*)\b/PTa$interfaceNames[$k]$1/g;
			}
			for (my $k = 0; $k <= $#otherNames; $k++)
			{
				$line =~ s/\bTa$otherNames[$k]_Ptr\b/PTa$otherNames[$k]/g;
			}
			
			$line =~ s/\bvoid_Ptr\b/PVoid/g;
			$line =~ s/\bint_Ptr\b/pInt/g;
			$line =~ s/\bint_Ptr\b/pInt/g;
			$line =~ s/\bta_u32_Ptr\b/pta_u32/g;
			$line =~ s/\bTaBool\b/TTaBool/g;
			$line =~ s/TTaPhysics\.dll/TaPhysics.dll/g;
		}
	
		print TMP_PAS $line;
	}
	close(TMP_PAS);	
	close(IN_PAS);
	
	open(OUT_PAS, ">$outDir"."Ta$outFile.pas"); 
	print OUT_PAS $pascalTmp;
	close(OUT_PAS);
		
}

sub replaceNames
{
#	for (my $k = 0; $k <= $#interfaceNames; $k++)
#	{
#		@_[0] =~ s/\b@interfaceNames[$k]\b/I@interfaceNames[$k]/g;
#	}
}
sub replaceNamesForC
{
	for (my $k = 0; $k <= $#interfaceNames; $k++)
	{
		@_[0] =~ s/\b@interfaceNames[$k]\b/Ta@interfaceNames[$k]/g;
	}
		
	for (my $k = 0; $k <= $#otherNames; $k++)
	{
		@_[0] =~ s/\b@otherNames[$k]\b/Ta@otherNames[$k]/g;
	}
	
	
	 @_[0] =~ s/\bbool\b/TaBool/g;
	
	 @_[0] =~ s/\bu8\b/ta_u8/g;
	 @_[0] =~ s/\bu16\b/ta_u16/g;
	 @_[0] =~ s/\bu32\b/ta_u32/g;
	 @_[0] =~ s/\bu64\b/ta_u64/g;
	 
	 @_[0] =~ s/\bs8\b/ta_s8/g;
	 @_[0] =~ s/\bs16\b/ta_s16/g;
	 @_[0] =~ s/\bs32\b/ta_s32/g;
	 @_[0] =~ s/\bs64\b/ta_s64/g;
}
sub hasReplacedName
{	
	for (my $k = 0; $k <= $#interfaceNames; $k++)
	{
		if (@_[0] =~ /Ta$interfaceNames[$k]/)
		{
			return 1;
		}
	}
	
	for (my $k = 0; $k <= $#otherNames; $k++)
	{
		if (@_[0] =~ /Ta$otherNames[$k]/)
		{
			return 1;
		}
	}
	
	if (@_[0] =~ /TaBool/ ||
		@_[0] =~ /ta_u32/)
	{
			return 1;
	}
	 
	return 0;	
}
sub dontUsePascalRef
{	
	for (my $k = 0; $k <= $#interfaceNames; $k++)
	{
		if (@_[0] =~ /Ta$interfaceNames[$k]/)
		{
			return 1;
		}
	}
	if (@_[0] =~ /void_Ptr/ ||
		@_[0] =~ /int_Ptr/ ||
		@_[0] =~ /ta_u32_Ptr/)
	{
		return 1;
	}
	return 0;
}
sub replaceNamesForPascal
{
	 @_[0] =~ s/\bunsigned int\b/Cardinal/g;
	 @_[0] =~ s/\bint\b/Integer/g;
	 @_[0] =~ s/\bfloat\b/Single/g;	
}
sub paramNeedsCast
{
	for (my $k = 0; $k <= $#interfaceNames; $k++)
	{
		if (@_[0] =~ /\b$interfaceNames[$k]\b/)
		{
			return 1;
		}
	}
	
	for (my $k = 0; $k <= $#otherNames; $k++)
	{
		if (@_[0] =~ /\b$otherNames[$k]\b/)
		{
			return 1;
		}
	}
	return 0;
}
